use crate::config::SMTPConfig;
use axum::extract::State;
use axum::http::StatusCode;
use axum::response::IntoResponse;
use axum::Json;
use lettre::message::header::ContentType;
use lettre::transport::smtp::authentication::Credentials;
use lettre::{Message, SmtpTransport, Transport};
use serde::Deserialize;
use std::fmt::Debug;
use std::str::FromStr;
use tracing::{error, info, instrument, warn};

#[derive(Debug, Deserialize)]
pub struct Registration {
    email_address: String,
}

#[instrument(skip(mail, smtp))]
pub async fn register(
    State(smtp): State<SMTPConfig>,
    Json(mail): Json<Registration>,
) -> impl IntoResponse {
    let email = match email_address::EmailAddress::from_str(&mail.email_address) {
        Ok(m) => m,
        Err(e) => {
            let err = format!("Invalid email address: {e}");
            warn!(email = mail.email_address, error = %err);
            return (StatusCode::BAD_REQUEST, err);
        }
    };
    info!(email = mail.email_address, "RegistrationRequest received");

    #[allow(clippy::unwrap_used)]
    let verification_email = Message::builder()
        .from(
            format!("Digitaler Frieden <{}>", smtp.mail())
                .parse()
                .unwrap(),
        )
        .to(email.to_string().parse().unwrap())
        .subject("Verify your email address")
        .header(ContentType::TEXT_HTML)
        .body(String::from("<h1>Verify your Email Address!</h1>"))
        .unwrap();

    let credentials = Credentials::new(smtp.mail().to_string(), smtp.secret().to_string());

    #[allow(clippy::unwrap_used)]
    let mailer = SmtpTransport::relay(smtp.provider())
        .unwrap()
        .credentials(credentials)
        .build();

    match mailer.send(&verification_email) {
        Ok(_) => {
            info!(email = mail.email_address, "Verification email sent");
            (StatusCode::OK, "Verification email sent".to_string())
        }
        Err(e) => {
            let err = format!("Failed to send verification email: {e}");
            error!(error = %err);
            (StatusCode::INTERNAL_SERVER_ERROR, err)
        }
    }
}
